#!/usr/bin/env python3

from argparse import ArgumentParser
from math import floor, log10
from timeit import default_timer as timer


def format_duration_with_prefix(duration, sig=2):
    # Round to significant digits
    duration = round(duration, -int(floor(log10(abs(duration)))) + (sig - 1))

    if duration < 1e-6:
        return f"{duration*1e9:.1f} ns"
    elif duration < 1e-3:
        return f"{duration*1e6:.1f} μs"
    elif duration < 1:
        return f"{duration*1e3:.1f} ms"
    else:
        return f"{duration:.1f} s"


def merge(left, right):
    combined = []

    i, j = 0, 0
    for k in range(len(left) + len(right)):
        if j >= len(right) or (i < len(left) and left[i] < right[j]):
            combined.append(left[i])
            i += 1
        else:
            combined.append(right[j])
            j += 1

    return combined


def merge_sort(array):
    if len(array) <= 1:
        return array

    left = merge_sort(array[: len(array) // 2])
    right = merge_sort(array[len(array) // 2 :])

    return merge(left, right)


def main():

    argparser = ArgumentParser("Run MergeSort on given file.")
    argparser.add_argument(
        "--time", action="store_true", help="Crudely report the running time."
    )
    argparser.add_argument("filename", help="Name of file to open and sort.")

    args = argparser.parse_args()

    # Open the file, read the contents, convert all values to ints
    with open(args.filename, "r") as input_file:
        n = int(input_file.readline())
        array = [int(v) for v in input_file.readlines()]

    # Run and time merge_sort
    start_time = timer()
    from_merge_sort = merge_sort(array)
    end_time = timer()

    # Check if list was sorted
    correctly_sorted = all(
        [x <= y for x, y in zip(from_merge_sort, from_merge_sort[1:])]
    )
    print("Sorted" if correctly_sorted else "No sorted")
    if args.time:
        print("Elapsed time:", format_duration_with_prefix(end_time - start_time))


if __name__ == "__main__":
    main()
